/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * This js file is for the script mediator specific UI operations
 */


collectdata["script"] = function collectlogdata(mediator) {
    var type = document.getElementById("type").value;

    var lang_ele = document.getElementById("language");
    var lang_index = lang_ele.selectedIndex;
    var language = lang_ele.options[lang_index].value;


    if (type == "inline") {
        var source_script = document.getElementById("source").value;
        if (language != "" && source_script != "") {
            if (language == "Javascript") {
                language = "js";
            }
            if (language == "Ruby") {
                language = "rb";
            }
            if (language == "Groovy") {
                language = "groovy";
            }
            mediator.setAttribute("language", language);
            var source = document.getElementById("source").value;
            if (mediator.firstChild != null) {
                mediator.removeChild(mediator.firstChild);
            }
            try {
                if (browsername.indexOf(IE_BROWSER_NAME) != -1) {
                    var doc = new ActiveXObject("Microsoft.XMLDOM");
                    doc.async = "false";
                    var sourceScript = doc.createCDATASection(source);
                    if (sourceScript != undefined && sourceScript != null) {
                        mediator.appendChild(sourceScript);
                    }
                } else if (browsername.indexOf(NETSCAPE_BROWSER_NAME) != -1) {
                    var parser = new DOMParser();
                    var doc = parser.parseFromString("", "text/xml");
                    var sourceScript = doc.createCDATASection(source);
                    if (sourceScript != undefined && sourceScript != null) {
                        mediator.appendChild(sourceScript);
                    }
                }
            } catch(e) {
                esbwarning("Error occurred when adding source script : " + e, TYPE_ERROR);
                return false;
            }
            if (mediator.getAttribute("key") != null) {
                mediator.removeAttribute("key");
            }
            if (mediator.getAttribute("function") != null) {
                mediator.removeAttribute("function");
            }
        } else {
            esbwarning("Language and Source Script must be specified", TYPE_INFO);
            return false;
        }
    }

    if (type == "regKey") {
        var key = document.getElementById("key").value;
        var func = document.getElementById("function").value;
        if (language != "" && key != "") {
            mediator.setAttribute("language", language);
            mediator.setAttribute("key", key);

            if (func != "") {
                mediator.setAttribute("function", func);
            } else {
                esbwarning("Function Attribute must be specified", TYPE_INFO);
                return false;
            }
            if (mediator.firstChild != null) {
                mediator.removeChild(mediator.firstChild);
            }
        } else {
            esbwarning("Language and Registry key must be specified", TYPE_INFO);
            return false;
        }
    }
    return true;
};


function changeSrcType(position) {


    if (document.getElementById("type").value == "inline") {
        setNonInlineEdit(position);

    }

    if (document.getElementById("type").value == "regKey") {
        setInlineEdit(position);
    }

}


function setNonInlineEdit(position) {

    var epdiv = document.getElementById("script_edit_div");
    var mediator = getnestedmediator(internalmodel.sequence, position);

    if (epdiv != null && mediator != null) {
        mediator.removeAttribute("noninline");
        mediator.setAttribute("inline", "1");
        processXML(mediator, "mediators/script_edit.xsl", epdiv);
    }

}
function setInlineEdit(position) {

    var epdiv = document.getElementById("script_edit_div");
    var mediator = getnestedmediator(internalmodel.sequence, position);
    if (epdiv != null && mediator != null) {
        mediator.removeAttribute("inline");
        mediator.setAttribute("noninline", "1");
        processXML(mediator, "mediators/script_edit.xsl", epdiv);
    }

}

function scriptkeyfromregistry(index) {
    var panelId = ("mypanel" + index).toString();
    registrybrowser("changeKeyVal", "dicardregistrybrowser", index, 1, panelId);
}

function changeKeyVal(position) {
    var mediator = getnestedmediator(internalmodel.sequence, position);
    var regKey = getRegistryKey()
    var divcontrol = document.getElementById("registryPanel");
    divcontrol.innerHTML = "";
    if (regKey != null && regKey != undefined) {
        mediator.setAttribute("key", regKey);
    }
    var epdiv = document.getElementById("script_edit_div");
    if (epdiv != null && mediator != null) {
        processXML(mediator, "mediators/script_edit.xsl", epdiv);
    }
}


function changeFunctionVal(position) {
    var mediator = getnestedmediator(internalmodel.sequence, position);
    var func = document.getElementById("function").value;
    if (func != "") {
        mediator.setAttribute("function", func);
    }
}

function changeSrcVal(position) {
    var source = document.getElementById("source").value;
    var mediator = getnestedmediator(internalmodel.sequence, position);
    if (mediator.firstChild != null) {
        mediator.removeChild(mediator.firstChild);
    }
    try {
        if (browsername.indexOf(IE_BROWSER_NAME) != -1) {
            var doc = new ActiveXObject("Microsoft.XMLDOM");
            doc.async = "false";
            var sourceScript = doc.createCDATASection(source);
            if (sourceScript != undefined && sourceScript != null) {
                mediator.appendChild(sourceScript);
            }
        } else if (browsername.indexOf(NETSCAPE_BROWSER_NAME) != -1) {
            var parser = new DOMParser();
            var doc = parser.parseFromString("", "text/xml");
            var sourceScript = doc.createCDATASection(source);
            if (sourceScript != undefined && sourceScript != null) {
                mediator.appendChild(sourceScript);
            }
        }
    } catch(e) {
        esbwarning("Error occurred when adding source script : " + e, TYPE_ERROR);
    }
}

function changeLanguageVal(position) {
    var lang_ele = document.getElementById("language");
    var lang_index = lang_ele.selectedIndex;
    var language = lang_ele.options[lang_index].value;
    var mediator = getnestedmediator(internalmodel.sequence, position);
    mediator.setAttribute("language", language)
}
