/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * This js is for sequence editing and all the management tasks related to sequences
 */

/* Global variable to store the sequence administration service */
var SEQUENCE_ADMIN = "SequenceAdmin";

/* Global object to hold the sequence internally */
var internalmodel = new Object();
internalmodel.sequence = "";
internalmodel.sequencename = "";
internalmodel.rolebackname = "";

/* Global object to hold the data collecting function literals as an associative array */
var collectdata = new Object();
/* Global object to hold the initialization of mediator nodes for specific tasks */
var initedit = new Object();
/* Global object to hold the initial mediator creation function literals as an associative array */
var mediatorelementstring = new Object();

var yuimenu = null;

/**
 * This function will be called to get the Sequence data for all the available sequences for the
 * initial show off of the sequences table
 */
function showsequences() {

    var body_xml = '<ns1:sequenceDataRequest xmlns:ns1="http://org.apache.synapse/xsd">' +
                   '</ns1:sequenceDataRequest>';

    var callURL = serverURL + "/" + SEQUENCE_ADMIN + "/" + "sequenceData";
    send("sequenceData", body_xml, "", callURL, "", false, showsequencescallback);
}

/**
 * This is the callback for the showsequences method from which initially the sequences will be
 * displayed in to the console
 */
function showsequencescallback() {
    if (!onError()) {
        return;
    }
    callbackhelper("sequences.xsl", document.getElementById("divSequences"));

    new YAHOO.widget.Tooltip("tt", { context:"tooltip_img", text:"asdasdasdasd" });
}

/**
 * This function will be called to delete the sequences from the console. It will invoke the
 * web service call deleteSequence with the given sequence name
 */
function deletesequence(sequencename) {

    var body_xml = '<ns1:deleteSequence xmlns:ns1="http://org.apache.synapse/xsd">' +
                   '<ns1:sequenceName>' + sequencename + '</ns1:sequenceName>' +
                   '</ns1:deleteSequence>';

    var callURL = serverURL + "/" + SEQUENCE_ADMIN + "/" + "deleteSequence";
    send("deleteSequence", body_xml, "", callURL, "", false, commonsequencecallback);
}

/**
 * This is the callback function of the deletesequence function and it will be executed on complete
 * of the above function to redisplay the set of availabel sequences
 */
function commonsequencecallback() {
    if (!onError()) {
        return;
    }
    showsequences();
}

function enableseqstatistics(sequencename) {

    var body_xml = '<ns1:enableStatistics xmlns:ns1="http://org.apache.synapse/xsd">' +
                   '<ns1:sequenceName>' + sequencename + '</ns1:sequenceName>' +
                   '</ns1:enableStatistics>';

    var callURL = serverURL + "/" + SEQUENCE_ADMIN + "/" + "enableStatistics";
    send("enableStatistics", body_xml, "", callURL, "", false, commonsequencecallback);
}

function disableseqstatistics(sequencename) {

    var body_xml = '<ns1:disableStatistics xmlns:ns1="http://org.apache.synapse/xsd">' +
                   '<ns1:sequenceName>' + sequencename + '</ns1:sequenceName>' +
                   '</ns1:disableStatistics>';

    var callURL = serverURL + "/" + SEQUENCE_ADMIN + "/" + "disableStatistics";
    send("disableStatistics", body_xml, "", callURL, "", false, commonsequencecallback);
}

function enableseqtracing(sequencename) {

    var body_xml = '<ns1:enableTracing xmlns:ns1="http://org.apache.synapse/xsd">' +
                   '<ns1:sequenceName>' + sequencename + '</ns1:sequenceName>' +
                   '</ns1:enableTracing>';

    var callURL = serverURL + "/" + SEQUENCE_ADMIN + "/" + "enableTracing";
    send("enableTracing", body_xml, "", callURL, "", false, commonsequencecallback);
}

function disableseqtracing(sequencename) {

    var body_xml = '<ns1:disableTracing xmlns:ns1="http://org.apache.synapse/xsd">' +
                   '<ns1:sequenceName>' + sequencename + '</ns1:sequenceName>' +
                   '</ns1:disableTracing>';

    var callURL = serverURL + "/" + SEQUENCE_ADMIN + "/" + "disableTracing";
    send("disableTracing", body_xml, "", callURL, "", false, commonsequencecallback);
}

/*
 * This function will be called to graphically view the sequence specified by the sequencename
 * and the viewsequencecallback will be executed as the callback for the service invocation
 */
function viewsequence(sequencename) {

    var bodyxml = '<ns1:getSequence xmlns:ns1="http://org.apache.synapse/xsd">' +
                  '<ns1:sequenceName>' + sequencename + '</ns1:sequenceName>' +
                  '</ns1:getSequence>';

    var callurl = serverURL + "/" + SEQUENCE_ADMIN + "/" + "getSequence";
    send("getSequence", bodyxml, "", callurl, "", false, viewsequencecallback);
}

/**
 * This is the callback for the viewsequence function which invokes the sequence admin service
 * and this will call the viewsequencechanges function to get the html rendered
 */
function viewsequencecallback() {
    if (!onError()) {
        return;
    }

    internalmodel.sequence = getNodeFromPath("return", xhReq.responseXML).firstChild;
    internalmodel.sequencename = internalmodel.sequence.getAttribute("name");
    internalmodel.rolebackname = internalmodel.sequencename;

    viewsequencechanges();
}

/**
 * This function is used to add sequences to the existing list of sequences but only in the UI level
 * to save these in to the configuration one must explicitly save that with a name
 */
function addsequence() {
    
    internalmodel.sequencename = "";
    internalmodel.sequence = createesbelement(
            "<syn:sequence name='' xmlns:syn='http://ws.apache.org/ns/synapse'/>");
    internalmodel.rolebackname = "";

    viewsequencechanges();
}

/**
 * This will generate the html representation of the service result (sequence) or after modifying
 * the model corresponds to the sequence in the UI level
 */
function viewsequencechanges() {

    var objdiv = document.getElementById("divSequence");
    objdiv.innerHTML = "";

    var displayedname = "Define new Sequence";
    if (internalmodel.sequencename != "") {
        displayedname = "Sequence " + internalmodel.sequencename;
    }

    objdiv.innerHTML = "<div class='breadcrumb'><a href='#' onclick='javascript: "
            + "showsequences(); return false;'>Manage Sequences</a>&#160;&gt;&#160; " + displayedname + "</div>"
            + "<div class='page-main-title title-sequences'>"
            + displayedname + "</div><br/><br/>";

    var relativediv = document.createElement("div");
    relativediv.setAttribute("style", "_margin-left:0px;");
    getsequencedisplay(relativediv);        
    objdiv.appendChild(relativediv);

    var savepanel = document.createElement("div");
    savepanel.className = "sequence-desc";
    savepanel.innerHTML += "<div id='savediv'><table><tbody><tr>"
            + "<td><input type='button' class='panelbutton_default button-space' "
            + "onclick='javascript: sequencesave(); return false;' value='Save'>"
            + "<input type='button' class='panelbutton_next button-space' "
            + "onclick='javascript:sequencesaveas(); return false;' value='Save As'/>"
            + "<input type='button' class='panelbutton_next button-space-big' style='width:120px;'"
            + "onclick='javascript:savesequencetoregistry(); return false;' value='Registry Export'/>"
            + "<input type='button' class='panelbutton_cancel' "
            + "onclick='javascript: showsequences(); return false;' value='Cancel'/></td>"
            + "</tr></tbody></table></div>";
    objdiv.appendChild(savepanel);
    if (browsername.indexOf(IE_BROWSER_NAME) != -1) {
        var newdiv = document.createElement("div");
        newdiv.style.height = "220px";
        objdiv.appendChild(newdiv);
    }
    showOnlyOneMain(objdiv);
}

/**
 * This is a helper method to get the sequence displayed in a div. Need this seperated out for
 * reuse this in the annonymus case
 */
function getsequencedisplay(objdiv) {

    var anonymous = false;

    if(internalmodel.sequence.getAttribute("anonymous") != null && internalmodel.sequence.getAttribute("anonymous") == "true") {
        anonymous = true;
    }

    objdiv.setAttribute("id", "displaydiv");

    if (internalmodel.sequence.firstChild == null) {
        objdiv.innerHTML += "<div class='mediator-add' id='mediator-add0'>"
                + "<img align='top' src='extensions/core/images/add_mediator.gif' "
                + "onclick='javascript:showaddmediator(event, 0);return false;'/>"
                + "<label class='styled-link' onclick='javascript:showaddmediator(event, 0);return false;'> Add Mediator </label></div>";
        objdiv.appendChild(document.createElement("br"));
    }

    var sequence_div = document.createElement("div");
    setnodepositions(internalmodel.sequence);
    processXML(internalmodel.sequence, "mediators/mediator_view.xsl", sequence_div);
    objdiv.appendChild(sequence_div);

    var onerrordiv = document.createElement("div");
    onerrordiv.setAttribute("id", "onerrordiv");
    var errorval;
    if(internalmodel.sequence.getAttribute("onError") != null) {
        errorval = internalmodel.sequence.getAttribute("onError");
    } else {
        errorval = "";
    }
    onerrordiv.innerHTML = "<label style='margin-left: 20px;'>On error sequence</label><input type='text' disabled='true' style='margin-left: 10px;' "
                + "class='esb-edit_disabled' value='" + errorval + "'/>"
                + "<img src='extensions/core/images/edit.gif' style='margin-bottom:-7px; margin-left:10px;'"
                + "onclick='javascript: editonerror(); return false;'/><img src='extensions/core/images/delete.gif' style='margin-bottom:-7px; margin-left:10px;'"
                + "onclick='javascript: resetonerror(); return false;'/><br/><br/>";
    if(!anonymous) {
        objdiv.appendChild(onerrordiv);
    }
}

function editonerror() {

    vieweditedmediator();

    var onerrordiv = document.getElementById("onerrordiv");

    var errorsequence = internalmodel.sequence.getAttribute("onError");
    if(errorsequence == null) {
        errorsequence = "";
    }

    if(onerrordiv != undefined) {
        onerrordiv.innerHTML = "";
        onerrordiv.innerHTML = "<label style='margin-left:50px;'>On error sequence </label>"
            + "<input id='on_error' style='margin-left: 50px;' type='text' size='20' "
            + "class='esb-edit_disabled' disabled='true' value='"
            + errorsequence + "'/>"
            + "<img style='margin-bottom:-7px;margin-left: 10px;' src='extensions/core/images/registry_picker.gif' "
            + "title='Registry Browser' onclick='javascript:onerrorfromregistry();return false;'/>"
            + "<image src='extensions/core/images/update.gif' style='margin-bottom:-7px; margin-left: 50px;' "
            + "value='Update' onclick='javascript: updateonerror(); return false;'/>"
            + "<img src='extensions/core/images/delete.gif' style='margin-bottom:-7px; margin-left:10px;'"
            + "onclick='javascript: resetonerror(); return false;'/><br/><br/>";
    }
}

function updateonerror() {

    var errorseq = document.getElementById("on_error");
    if(errorseq != null && errorseq.value != "") {
        internalmodel.sequence.setAttribute("onError", errorseq.value);
    }

    reloadsequencedisplay();
}

function resetonerror() {
    internalmodel.sequence.removeAttribute("onError");
    reloadsequencedisplay();
}

function onerrorfromregistry() {

    registrybrowser("setonerrorfromregistry", "dicardregistrybrowser", "", 1, "registryPanel");
}

function setonerrorfromregistry() {

    internalmodel.sequence.setAttribute("onError", getRegistryKey());

    dicardregistrybrowser(1);
    
    viewsequencechanges();
    editonerror();

}

/**
 * This is a helper function to encapsulate the id of the div and call the getsequencedisplay
 * method in to a separate div
 */
function reloadsequencedisplay() {

    var relativediv = document.getElementById("displaydiv");
    relativediv.innerHTML = "";
    getsequencedisplay(relativediv);
}

/**
 * This function is used to generate the UI when a portion of the sequence is updated from editing
 * the sequence (mediator update)
 */
function viewmediator(mediatordiv, mediatornode, position) {
    
    setnodepositions(mediatornode, position);
    //    showStuffInNewWindow(new XMLSerializer().serializeToString(mediatornode));
    mediatordiv.innerHTML = "";
    processXML(mediatornode, "mediators/mediator_view.xsl", mediatordiv);
}

/**
 * This is an internaly used function to set the node positions to the sequence dom for the as a
 * method of parsing position argument with each and every node to the xslt
 */
function setnodepositions(mediatornode, position) {

    if (mediatornode != null &&
        typeof(mediatornode) != undefined && typeof(mediatornode.setAttribute) != "undefined") {

        var mediatorchilds = mediatornode.childNodes;

        var fragment = "";
        if (position != undefined) {
            mediatornode.setAttribute("esb_med_pos", position);
            fragment += position += ".";
        }

        for (var i = 0; i < mediatorchilds.length; i++) {
            setnodepositions(mediatorchilds[i], fragment + i);
        }
    }
}

function clearnodepositions(mediatornode) {
    if (mediatornode.nodeType != 3) {
        mediatornode.removeAttribute("esb_med_pos");
        var med_children = mediatornode.childNodes;
        for (var i = 0; i < med_children.length; i++) {
            clearnodepositions(med_children[i]);
        }
    }
}

/**
 * This function is used to generate the edit view of the corresponding mediator node in the
 * internal model according to the position. This function will use the mediator nodename to
 * generate the xslt file name for the rendering
 */
function editmediator(position) {

    if (position != geteditingmediatorposition() && geteditingmediatorposition() != -1) {
        if (window.confirm("You haven't saved your changes. Do you wish to keep any changes?")) {
            if(!vieweditedmediator()) {
                return;
            }
        } else {
            var editingpos = geteditingmediatorposition();
            var mediatorDiv = document.getElementById("mediatordiv" + editingpos).parentNode;
            viewmediator(mediatorDiv,
                    getnestedmediator(internalmodel.sequence, editingpos), editingpos);
        }
    }
    
    var mediatorDiv = document.getElementById("mediatordiv" + position);

    var editdiv = document.createElement("div");
    editdiv.setAttribute("id", "editingDiv");
    mediatorDiv.innerHTML = "";
    mediatorDiv.appendChild(editdiv);

    var actionpaneldiv = document.getElementById("nested-mediator-actions" + position);
    var newlink = actionpaneldiv.getElementsByTagName("a")[0];
    newlink.onclick = vieweditedmediator;
    newlink.getElementsByTagName("img")[0].setAttribute("src", "extensions/core/images/update.gif");
    newlink.setAttribute("title", "Update");

    var mediator = getnestedmediator(internalmodel.sequence, position);
    var mediatorname = mediator.nodeName.substring(mediator.nodeName.indexOf(":") + 1, mediator.nodeName.length);
    var xsltfilename = "mediators/" + mediatorname + "_edit.xsl";

    if (initedit[mediatorname] != undefined) {
        initedit[mediatorname](mediator);
    }

     processXML(mediator, xsltfilename, editdiv);

}

/**
 * This is an internal function for getting the mediator element from the given tree of mediators
 * in the dom defined in mediatornode with the depth described in position
 */
function getnestedmediator(mediatornode, position) {

    position = position.toString();
    if (position.indexOf(".") == -1) {
        if(mediatornode != undefined && mediatornode.childNodes.length != 0)
            return mediatornode.childNodes[position];
    } else {
        return getnestedmediator(mediatornode.childNodes[position.substring(0, ("" + position)
                .indexOf("."))], position.substring(position.indexOf(".") + 1, position.length));
    }
}

/*
 * Calls the relevent function to collect data depending on the element in the coresponding
 * sequence internal model and view the edited mediator in the console
 */
function vieweditedmediator() {

    var position = geteditingmediatorposition();

    if (position != -1) {

        var mediatorname = getnestedmediator(internalmodel.sequence, position).nodeName;

        // calling the relevant function in the acssociative array - *****variable function calling
        // to collect data for respective mediators and modify the internal model
        if (collectdata[mediatorname.substring(mediatorname.indexOf(":") + 1, mediatorname.length)] == null) {
            if (!collectdata["mediator"](position)) {
                return false;
            }
        } else {
            if(!collectdata[mediatorname.substring(mediatorname.indexOf(":") + 1,
                    mediatorname.length)](getnestedmediator(internalmodel.sequence, position))) {
                return false;
            }
        }

        var mediatorDiv = document.getElementById("mediatordiv" + position).parentNode.parentNode;
        viewmediator(mediatorDiv, getnestedmediator(internalmodel.sequence, position), position);
    }

    return true;
}

/*
 * This method deletes the mediator in the given position of the sequence. A wrapper method for the
 * nesteddelete internal method
 */
function deletemediator(position) {

    var edit_pos = geteditingmediatorposition();
    if (edit_pos != -1 && edit_pos != position && edit_pos.indexOf(position) != 0) {
        vieweditedmediator();
    }

    nesteddelete(internalmodel.sequence, position);
    if (edit_pos != -1 && edit_pos != position && position && edit_pos.indexOf(position) != 0) {
        editmediator(edit_pos)
    }
}
    

/**
 * This method deletes the mediator in the given position of the given node described as
 * mediatornode which is either a mediator or a sequence
 */
function nesteddelete(mediatornode, position) {

    if (position.indexOf(".") == -1) {
        mediatornode.removeChild(mediatornode.childNodes[position]);
        reloadsequencedisplay();
    } else {
        nesteddelete(mediatornode.childNodes[position.substring(0, ("" + position)
                .indexOf("."))], position.substring(position.indexOf(".") + 1, position.length));
    }
}

/*
 * Displays a YUI menu to provide necessary parameters to add a new mediator
 */
function showaddmediator(event, position) {

    if (yuimenu == null) {

        yuimenu = new YAHOO.widget.Menu("yuimenu", {});

    }

    yuimenu.clearContent();
    var menuItems = yuimenu.addItems(mediatorcategories);

    for (var i = 0; i < menuItems.length; i++) {
        var submenu = menuItems[i].cfg.getProperty("submenu").getItems();
        for (var j = 0; j < submenu.length; j++) {
            submenu[j].clickEvent.subscribe(addmediator, {position: position});
        }
    }


    yuimenu.render(document.getElementById("divOverMenu"));
    document.getElementById("divOverMenu").style.left = event.clientX;
    document.getElementById("divOverMenu").style.top = document.body.scrollTop + event.clientY;
    document.getElementById("divOverMenu").style.display = "inline";
    yuimenu.show();
}

/*
 * Collects the data for the new mediator and adds a new mediator to the existing sequence in the UI
 * level for this addition to be activated in to the configuration one must save the sequence
 */
function addmediator(mediatoritem, mediatorargs, mediatorvalue) {

    if (geteditingmediatorposition() != -1) {
        if (window.confirm("You haven't saved your changes. Do you wish to keep any changes?")) {
            vieweditedmediator();
        } else {
            var editingpos = geteditingmediatorposition();
            var mediatorDiv = document.getElementById("mediatordiv" + editingpos).parentNode;
            viewmediator(mediatorDiv,
                    getnestedmediator(internalmodel.sequence, editingpos), editingpos);
        }
    }

    var mediatortypevalue = this.id;

    var mediatorstring;
    if (mediatorelementstring[mediatortypevalue]) {
        mediatorstring = mediatorelementstring[mediatortypevalue]();
    } else {
        mediatorstring = "<syn:" + mediatortypevalue + " xmlns:syn='" + SYNAPSE_NS + "'/>";
    }

    var mediatorelement = getnestedelement(internalmodel.sequence, mediatorvalue.position, mediatorstring);

    reloadsequencedisplay();
    var posstring = mediatorvalue.position.toString();
    var editposition;

    if(posstring.lastIndexOf(".") != -1) {
        editposition = (posstring.substring(0, posstring.lastIndexOf(".") + 1) + editwhenadd_lastpos).toString();
    } else {
        editposition = editwhenadd_lastpos.toString();
    }
    // Edit disabled for the "in", "out" and "drop" mediators
    if(mediatortypevalue != "in" && mediatortypevalue != "out" && mediatortypevalue != "drop") {
        editmediator(editposition);
    }

}

var editwhenadd_lastpos;

/**
 * This is an internal method for getting the mediator node described in mediatornode modified as
 * added the element described in mediatorstring to the position in the mediatornode
 */
function getnestedelement(mediatornode, position, mediatorstring) {

    position = position.toString();
    if (position.indexOf(".") != -1) {

        var newpos = position.substring(position.indexOf(".") + 1, position.length);
        var medpos = position.substring(0, position.indexOf("."));

        mediatornode.replaceChild(getnestedelement(mediatornode
                .childNodes[medpos], newpos, mediatorstring), mediatornode.childNodes[medpos]);
        return mediatornode;
    } else {

        var newnode = createesbelement(mediatorstring);

        if (mediatornode.childNodes.length - 1 == position) {
            mediatornode.appendChild(newnode);
            editwhenadd_lastpos = mediatornode.childNodes.length - 1;
        } else {
            mediatornode.insertBefore(newnode, mediatornode.childNodes[parseInt(position) + 1]);
            if(mediatornode.childNodes.length == 1) {
                editwhenadd_lastpos = "0";
            } else {
                editwhenadd_lastpos = parseInt(position) + 1;
            }
        }

        return mediatornode;
    }
}

/*
 * Moves the mediator which is calling the method to the upper position and swap them
 * in the internal model and then reload the document using the xslt to generate the UI
 */
function movemediatorup(position) {

    var editingposition = geteditingmediatorposition();
    var editmovement = parseInt(position) - 1;

    if(editingposition != -1) {
        var editingelem = getnestedmediator(internalmodel.sequence, editingposition);

        collectdata[editingelem.nodeName.substring(editingelem.nodeName.indexOf(":") + 1, editingelem.nodeName.length)] (editingelem, true);
    }

    nestedmoveup(position, internalmodel.sequence);
    reloadsequencedisplay();

    if (position.indexOf(".") != -1) {
        editmovement = position.substring(0, position.lastIndexOf(".") + 1)
                + (parseInt(position.substring(position.lastIndexOf(".") + 1, position.length)) - 1);
    }

    // reedit the editing mediator which was editing before the move
    if (editingposition != -1) {
        if (editingposition == position) {
            editmediator(editmovement);
        } else if (editingposition == editmovement) {
            editmediator(position);
        } else {
            editmediator(editingposition);
        }
    }
}

/**
 * This is an internal method used by the movemediatorup method to do the real moving part of the
 * internal model before viewing it.
 */
function nestedmoveup(position, mediatornode) {

    if (position.indexOf(".") == -1) {
        var movingchild = mediatornode.childNodes[position].cloneNode(1);
        var movedchild = mediatornode.childNodes[position - 1].cloneNode(1);
        mediatornode.replaceChild(movedchild, mediatornode.childNodes[position]);
        mediatornode.replaceChild(movingchild, mediatornode.childNodes[position - 1]);
    } else {
        nestedmoveup(position.substring(position.indexOf(".") + 1, position.length),
                mediatornode.childNodes[position.substring(0, position.indexOf("."))]);
    }
}

/*
 * Moves the mediator which is calling the method to the lower position and swap them
 * in the internal model and then reload the document using the xslt to generate the UI
 */
function movemediatordown(position) {

    var editingposition = geteditingmediatorposition();
    var editmovement = parseInt(position) + 1;

    if(editingposition != -1) {
        var editingelem = getnestedmediator(internalmodel.sequence, editingposition);

        collectdata[editingelem.nodeName.substring(editingelem.nodeName.indexOf(":") + 1, editingelem.nodeName.length)] (editingelem, true);
    }
    
    nestedmovedown(position, internalmodel.sequence);
    reloadsequencedisplay();

    if (position.indexOf(".") != -1) {
        editmovement = position.substring(0, position.lastIndexOf(".") + 1)
                + (parseInt(position.substring(position.lastIndexOf(".") + 1, position.length)) + 1);
    }

    // reedit the editing mediator which was editing before the move
    if (editingposition != -1) {
        if (editingposition == position) {
            editmediator(editmovement);
        } else if (editingposition == editmovement) {
            editmediator(position);
        } else {
            editmediator(editingposition);
        }
    }
}

/**
 * This is an internal method used by the movemediatordown method to do the real moving part of the
 * internal model before viewing it.
 */
function nestedmovedown(position, mediatornode) {

    if (position.indexOf(".") == -1) {
        var movingchild = mediatornode.childNodes[position].cloneNode(1);
        var movedchild = mediatornode.childNodes[parseInt(position) + 1].cloneNode(1);
        mediatornode.replaceChild(movedchild, mediatornode.childNodes[position]);
        mediatornode.replaceChild(movingchild, mediatornode.childNodes[parseInt(position) + 1]);
    } else {
        nestedmovedown(position.substring(position.indexOf(".") + 1, position.length),
                mediatornode.childNodes[position.substring(0, position.indexOf("."))]);
    }
}

/**
 * This method validates the sequence and call the web service to save the sequence to the
 * configuration after doing the editing in the UI level
 */
function sequencesave() {

    if(!vieweditedmediator()) {
        return;
    }
    updateonerror();

    if (internalmodel.sequencename == "") {
        showgetnamediv(0);
    } else {
        internalmodel.rolebackname = internalmodel.sequencename;
        var sequencestr = getmediatorstring(internalmodel.sequence);
        var body_xml = '<ns1:saveSequence xmlns:ns1="http://org.apache.synapse/xsd">' +
                       '<ns1:sequenceElement>' + relpaceString(relpaceString(trim(sequencestr), "&amp;", "&"), "&", "&amp;") + '</ns1:sequenceElement>' +
                       '</ns1:saveSequence>';

        var callURL = serverURL + "/" + SEQUENCE_ADMIN + "/" + "saveSequence";
        send("saveSequence", body_xml, "", callURL, "", false, sequencesavecallback);
    }
}

/**
 * This is the callback of the savesequence method which will load the set of existing sequences to
 * the main console on complete of the savesequence method
 */
function sequencesavecallback() {
    if (!onError()) {
        return;
    }
    showsequences();
}

/**
 * This is an internal method for generating the html fragment for getting the sequence name as an
 * input from the console when the sequence needs a one
 */
function showgetnamediv(type) {

    var savediv = document.getElementById("savediv");
    savediv.innerHTML = "<label class='getname'> Specify the sequence name </label>"
            + "<input type='text' id='sequencename' class='esb-edit' value='" + internalmodel.sequencename + "'/>"
            + "<br/><br/><table style='margin-left: 150px;'><tbody><tr><td>"
            + "<input type='button' class='panelbutton_default' "
            + "onclick='javascript:getvalueofname(" + type + "); return false;' value='Save'/></td>"
            + "<td></td><td><input type='button' class='panelbutton_cancel' "
            + "onclick='javascript: rolebacksequence(); return false;' value='Cancel'/></td>"
            + "</tr></tbody></table>";
}

/**
 * This is an internal method for getting the name for the sequence from the above control
 * displayed in to the console using the showgetnamediv and assign that to the sequence
 */
function getvalueofname(type) {

    if(namevalidate(document.getElementById("sequencename").value) && document.getElementById("sequencename").value != "") {

        internalmodel.sequencename = document.getElementById("sequencename").value;
        internalmodel.sequence.setAttribute("name", internalmodel.sequencename);

        vieweditedmediator();

        if (type == 0) {
            addsequencesave();
        } else {
            sequencesave();
        }
    } else if(document.getElementById("sequencename").value == "") {
        esbwarning("Name must be specified", TYPE_INFO);
    } else {
        esbwarning("Invalid characters present in the name", TYPE_WARN);
    }
}

/**
 * This method will be called when a new sequence will be added to the synapse configuration and
 * this will makesure that the newly added sequence will not delete an existing one
 */
function addsequencesave() {

    if (internalmodel.sequencename == "") {
        showgetnamediv(0);
    } else {

        var sequencestr = getmediatorstring(internalmodel.sequence);
        var body_xml = '<ns1:addSequence xmlns:ns1="http://org.apache.synapse/xsd">' +
                       '<ns1:sequenceElement>' + relpaceString(relpaceString(trim(sequencestr), "&amp;", "&"), "&", "&amp;") + '</ns1:sequenceElement>' +
                       '</ns1:addSequence>';

        var callURL = serverURL + "/" + SEQUENCE_ADMIN + "/" + "addSequence";
        send("addSequence", body_xml, "", callURL, "", false, addsequencesavecallback);
    }
}

/**
 * This is the callback function of the addsavesequence function and this will be executed on
 * complete of the above method to show the existing sequences after the new addition
 */
function addsequencesavecallback() {
    if (!onError()) {
        internalmodel.sequencename = "";
        return;
    }
    showsequences();
}

// todo: do we need to provide the functionality of overriding a sequence over deleting and
// todo: recreating the sequence
/**
 * This method will be used to save an existing sequence with a different name to the synapse
 * configuration for future uses (clone)
 */
function sequencesaveas() {
    internalmodel.rolebackname = internalmodel.sequencename;
    internalmodel.sequencename = "";
    sequencesave();
}

function savesequencetoregistry() {
    var med_pos_edit = geteditingmediatorposition();
    if (med_pos_edit != -1) {
        var med = getnestedmediator(internalmodel.sequence, med_pos_edit);
        collectdata[med.localName](med, true);
    }
    registrybrowserforsave("sequencetoregistry", "dicardregistrybrowser", "", "", "savetoregistryPanel");
}

var sequencecacheableduration;
var fullFilePath;

function sequencetoregistry() {
    internalmodel.sequence.setAttribute("name", "anonymous");
    var sequencetobesaved = getmediatorstring(internalmodel.sequence);
    var sequencenametosave = document.getElementById("regKeyField").value;
    sequencecacheableduration = document.getElementById("cacheDurField").value;
    if (sequencecacheableduration != "" && isNaN(sequencecacheableduration)) {
        esbwarning("Cachable duration (Seconds) should be a valid number", TYPE_WARN);
        return;
    }
    var parentFolder = "";

    if (filetree.getSelected() != null) {
        parentFolder = filetree.getSelected();
        if (parentFolder != "") {
            fullFilePath = parentFolder + "/" + sequencenametosave;
        }
    }

    var body_xml = '<ns1:createFile xmlns:ns1="http://org.apache.synapse/xsd">' +
                   '<file>' +
                   '<parent>' + parentFolder + '</parent>' +
                   '<fileName>' + sequencenametosave + '</fileName>' +
                   '<content>' + sequencetobesaved + "</content>" +
                   '</file>' +
                   '</ns1:createFile>';

//    alert(parentFolder)
//    alert(sequencenametosave)
//    alert(new XMLSerializer().serializeToString(sequencetobesaved))

    var callURL = serverURL + "/" + "RegistryAdmin" + "/" + "createFile";

    send("createFile", body_xml, "", callURL, "", false, finalizeregistrysave);
}

function finalizeregistrysave() {
    if (!onError()) {
        return;
    }
    if (sequencecacheableduration != "") {
        var body_xml = '<ns1:updateRegistryEntry xmlns:ns1="http://org.apache.synapse/xsd">' +
                       '<regEntry>' +
                       '<key>' + fullFilePath + '</key>' +
                       '<expiryTime>' + parseInt(sequencecacheableduration) + '</expiryTime>' +
                       '</regEntry>' +
                       '</ns1:updateRegistryEntry>';

        var callURL = serverURL + "/" + "RegistryAdmin" + "/" + "updateRegistryEntry";

        send("updateRegistryEntry", body_xml, "", callURL, "", false, finalizeEntryUpdate);
    } else {
        dicardregistrybrowser();
        showsequences();
    }
}

function finalizeEntryUpdate() {
    if (!onError()) {
        return;
    }
    dicardregistrybrowser();
    showsequences();
}

function showsequenceinregistry() {
    var sequence_div = document.createElement("div");
    sequence_div.style.height = 400;
    sequence_div.style.overflowY = "auto";
    sequence_div.style.overflowX = "hidden";
    getsequencedisplay(sequence_div);
    var temp = document.createElement("div");
    temp.appendChild(sequence_div);

    var bodyHTML = "<br/>"
            + "<div id='divSequence'>" + temp.innerHTML//new XMLSerializer().serializeToString(sequence_div)
            + "</div><div><table style='margin-left: 150px;'><tbody><tr>"
            + "<td><input type='button' class='panelbutton_default' "
            + "onclick='javascript: editRegistrySequenceFile(); return false;' value='Save'></td>"
            + "<td></td><td><input type='button' class='panelbutton_cancel' "
            + "onclick='javascript: cancelSeqEditor(); return false;' value='Cancel'/></td>"
            + "</tr></tbody></table></div>";

    yuieditorpanel("WSO2-ESB Sequence Editor", bodyHTML, "Anonymous sequence", "panelSequence", "800px");
}

function editRegistrySequenceFile() {

    var med_pos_edit = geteditingmediatorposition();
    if (med_pos_edit != -1) {
        var med = getnestedmediator(internalmodel.sequence, med_pos_edit);
        collectdata[med.localName](med, true);
    }

    var fileContent = getmediatorstring(internalmodel.sequence);
    editFilePath = regTree.getSelected();

    var body_xml = '<ns1:updateRegistryEntryValue xmlns:ns1="http://org.apache.synapse/xsd">' +
                   '<regEntry>' +
                   '<key>'+ regTree.getSelected() +'</key>' +
                   '<value>' + fileContent + '</value>' +
                   '</regEntry>' +
                   '</ns1:updateRegistryEntryValue>';

    var callURL = serverURL + "/" + "RegistryAdmin" + "/" + "updateRegistryEntryValue";

    send("updateRegistryEntryValue", body_xml, "", callURL, "", false, editRegistrySequenceFileCallback);

}

function editRegistrySequenceFileCallback() {
    if(!onError()) {
        return;
    }
    cancelSeqEditor();
    resetRegistryDisplay();
    regTree.fillAndExpand(editFilePath);
}

function cancelSeqEditor() {
    document.getElementById("panelSequence").innerHTML = "";
}

/**
 * This method is the roleback method when user clicks on the cancel button of the add new sequence
 * section and this will roleback the name of the sequence
 */
function rolebacksequence() {
    internalmodel.sequencename = internalmodel.rolebackname;
    viewsequencechanges();
}

/*
 * Retrieves the position of the editing mediator in the internal model
 */
function geteditingmediatorposition() {

    if (document.getElementById("editingDiv") != null) {

        var mediatordivid = document.getElementById("editingDiv").parentNode.getAttribute("id");
        return mediatordivid.substring(11, mediatordivid.length);
    } else {
        return -1;
    }
}

/**
 * This will return the string representation of the mediator node given, will be used internally
 */
function getmediatorstring(mediator) {

    if (mediator.nodeType == 3) {
        if (browsername.indexOf(NETSCAPE_BROWSER_NAME) != -1) {
            return mediator.textContent;
        }
        else if (browsername.indexOf(IE_BROWSER_NAME) != -1) {
            return mediator.text;
        }
    } else  if (mediator.nodeType == 4) {
        if (browsername.indexOf(NETSCAPE_BROWSER_NAME) != -1) {
            return new XMLSerializer().serializeToString(mediator);
        }
        else if (browsername.indexOf(IE_BROWSER_NAME) != -1) {
            return mediator.xml;
        }
    } else {
        var mediatorstring = '<' + mediator.nodeName;
        if (mediator.attributes) {
            for (var i = 0; i < mediator.attributes.length; i++) {
                if (mediator.attributes[i].name != "esb_med_pos") {
                    mediatorstring += ' '
                            + mediator.attributes[i].name + '="' + mediator.attributes[i].value + '"';
                }
            }
        }
        if (mediator.childNodes.length == 0) {
            mediatorstring += '/>';
        } else {
            mediatorstring += '>';
            for (var j = 0; j < mediator.childNodes.length; j++) {
                var chmedstr = getmediatorstring(mediator.childNodes[j]);
                if (chmedstr != null && chmedstr != undefined && chmedstr != "") {
                    mediatorstring += chmedstr;
                }
            }
            mediatorstring += '</' + mediator.nodeName + '>     ';
        }
        return mediatorstring;

    }
}