<!--
  ~ Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
  ~
  ~ Licensed under the Apache License, Version 2.0 (the "License");
  ~ you may not use this file except in compliance with the License.
  ~ You may obtain a copy of the License at
  ~
  ~      http://www.apache.org/licenses/LICENSE-2.0
  ~
  ~ Unless required by applicable law or agreed to in writing, software
  ~ distributed under the License is distributed on an "AS IS" BASIS,
  ~ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  ~ See the License for the specific language governing permissions and
  ~ limitations under the License.
  -->

<xsl:stylesheet version="1.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

    <xsl:include href="commons.xsl"/>
    <xsl:template match="syn:script" xmlns:syn="http://ws.apache.org/ns/synapse">
        <div id="script_edit_div">
            <label class="content-header">
                <img src="extensions/core/images/script-mediator.gif"/>
                <xsl:text>Script Mediator</xsl:text>
            </label>
            <br/>
            <br/>

            <xsl:variable name="nonEdit">
                <option value="inline">
                    <xsl:if test="not(@key)">
                        <xsl:attribute name="selected">true</xsl:attribute>
                    </xsl:if>
                    Inline
                </option>
                <option value="regKey">
                    <xsl:if test="@key">
                        <xsl:attribute name="selected">true</xsl:attribute>
                    </xsl:if>
                    Registry Key
                </option>
            </xsl:variable>

            <xsl:variable name="edit">
                <option value="inline">
                    <xsl:if test="@inline">
                        <xsl:attribute name="selected">true</xsl:attribute>
                    </xsl:if>
                    Inline
                </option>
                <option value="regKey">
                    <xsl:if test="@noninline">
                        <xsl:attribute name="selected">true</xsl:attribute>
                    </xsl:if>
                    Registry Key
                </option>
            </xsl:variable>

            <xsl:variable name="type">
                <tr>
                    <td>Script Type</td>
                    <td>
                        <select id="type" class="esb-edit" width="150px">
                            <xsl:attribute name="onchange">
                                javascript:changeSrcType('<xsl:value-of select="@esb_med_pos"/>');
                            </xsl:attribute>

                            <xsl:if test="not(@inline) and not(@noninline)">
                                <xsl:copy-of select="$nonEdit"/>
                            </xsl:if>
                            <xsl:if test="@inline or @noninline">
                                <xsl:copy-of select="$edit"/>
                            </xsl:if>
                        </select>
                    </td>
                </tr>
            </xsl:variable>

            <xsl:variable name="language">
                <tr>
                    <td>Language<font style='color: red;font-size: 8pt;'>&#160;*</font></td>
                    <td>
                        <select id="language" class="esb-edit" width="150px">
                            <xsl:attribute name="onchange">
                                javascript:changeLanguageVal('<xsl:value-of select="@esb_med_pos"/>');
                            </xsl:attribute>
                            <option value="js">
                                <xsl:if test="@language='js'">
                                    <xsl:attribute name="selected">true</xsl:attribute>
                                </xsl:if>
                                Javascript
                            </option>
                            <option value="rb">
                                <xsl:if test="@language='rb'">
                                    <xsl:attribute name="selected">true</xsl:attribute>
                                </xsl:if>
                                Ruby
                            </option>
                            <option value="groovy">
                                <xsl:if test="@language='groovy'">
                                    <xsl:attribute name="selected">true</xsl:attribute>
                                </xsl:if>
                                Groovy
                            </option>
                        </select>
                    </td>
                </tr>
            </xsl:variable>

            <xsl:variable name="source">
                <tr>
                    <td>
                        Source Script<font style='color: red;font-size: 8pt;'>&#160;*</font>
                    </td>
                    <td>
                        <textarea id="source" type="text" class="esb-edit" cols="60" rows="20"
                                  style="width:400; height:300px;">
                            <xsl:attribute name="onchange">
                                javascript:changeSrcVal('<xsl:value-of select="@esb_med_pos"/>');
                            </xsl:attribute>
                            <xsl:value-of select="."/>
                        </textarea>
                    </td>
                </tr>
            </xsl:variable>

            <xsl:variable name="key">
                <tr>
                    <td>
                        Key<font style='color: red;font-size: 8pt;'>&#160;*</font>
                    </td>
                    <td>
                        <input id="key" type="text" class="esb-edit_disabled" disabled="" width="150px">
                            <xsl:attribute name="onchange">
                                javascript:changeKeyVal('<xsl:value-of select="@esb_med_pos"/>');
                            </xsl:attribute>
                            <xsl:attribute name="value">
                                <xsl:value-of select="@key"/>
                            </xsl:attribute>
                            <xsl:attribute name="disabled">
                                true
                            </xsl:attribute>
                        </input>
                    </td>
                    <td>
                        <xsl:element name="img">
                            <xsl:attribute name="src">
                                extensions/core/images/registry_picker.gif</xsl:attribute>
                            <xsl:attribute name="title">Registry Browser</xsl:attribute>
                            <xsl:attribute name="onClick">
                                javascript:scriptkeyfromregistry('<xsl:value-of select="@esb_med_pos"/>');return false;
                            </xsl:attribute>
                        </xsl:element>
                    </td>
                </tr>
            </xsl:variable>
            <xsl:variable name="function">
                <tr>
                    <td>
                        Function<font style='color: red;font-size: 8pt;'>&#160;*</font>
                    </td>
                    <td>
                        <input id="function" type="text" class="esb-edit" width="150px">
                            <xsl:attribute name="onchange">
                                javascript:changeFunctionVal('<xsl:value-of select="@esb_med_pos"/>');
                            </xsl:attribute>
                            <xsl:attribute name="value">
                                <xsl:value-of select="@function"/>
                            </xsl:attribute>
                        </input>
                    </td>
                </tr>
            </xsl:variable>

            <xsl:if test="not(@inline or @noninline)">
                <xsl:if test="not(@key)">
                    <table border="0" style="margin-left: 3em; width: 300px;">
                        <xsl:copy-of select="$type"/>
                        <xsl:copy-of select="$language"/>
                        <xsl:copy-of select="$source"/>
                    </table>
                </xsl:if>


                <xsl:if test="@key">

                    <table border="0" style="margin-left: 3em; width: 300px;">
                        <xsl:copy-of select="$type"/>
                        <xsl:copy-of select="$language"/>
                        <xsl:copy-of select="$key"/>
                        <xsl:copy-of select="$function"/>
                    </table>
                </xsl:if>
            </xsl:if>

            <xsl:if test="@inline">
                <table border="0" style="margin-left: 3em; width: 700px;">
                    <xsl:copy-of select="$type"/>
                    <xsl:copy-of select="$language"/>
                    <xsl:copy-of select="$source"/>
                </table>
            </xsl:if>

            <xsl:if test="@noninline">
                <table border="0" style="margin-left: 3em; width: 300px;">
                    <xsl:copy-of select="$type"/>
                    <xsl:copy-of select="$language"/>
                    <xsl:copy-of select="$key"/>
                    <xsl:copy-of select="$function"/>
                </table>
            </xsl:if>

        </div>
    </xsl:template>
</xsl:stylesheet>