<!--
  ~ Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
  ~
  ~ Licensed under the Apache License, Version 2.0 (the "License");
  ~ you may not use this file except in compliance with the License.
  ~ You may obtain a copy of the License at
  ~
  ~      http://www.apache.org/licenses/LICENSE-2.0
  ~
  ~ Unless required by applicable law or agreed to in writing, software
  ~ distributed under the License is distributed on an "AS IS" BASIS,
  ~ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  ~ See the License for the specific language governing permissions and
  ~ limitations under the License.
  -->

<xsl:stylesheet version="1.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
    
    <xsl:include href="mediator_view.xsl"/>

    <xsl:template match="syn:validate" xmlns:syn="http://ws.apache.org/ns/synapse">
        <div id="validate_edit_div">
            <label class="content-header">
                <img src="extensions/core/images/validate-mediator.gif"/>
                <xsl:text>Validate Mediator</xsl:text>
            </label>
            <br/>
            <br/>
            <div id="script_edit" class="end-point-div">
                <br/>
                <xsl:if test="count(//syn:schema)!=0">
                    <p>Schema Keys defined for Validate Mediator</p>
                    <br/>
                    <table border="0" style="margin-left: 3em; width: 150px;">
                        <tbody>
                            <xsl:call-template name="schema_keys_template">
                                <xsl:with-param name="value">
                                    <xsl:value-of
                                            select="count(//syn:schema)"/>
                                </xsl:with-param>
                                <xsl:with-param name="limit">
                                    <xsl:value-of
                                            select="count(//syn:schema)"/>
                                </xsl:with-param>
                            </xsl:call-template>
                        </tbody>
                    </table>
                </xsl:if>
                <xsl:element name="input">
                    <xsl:attribute name="type">button</xsl:attribute>
                    <xsl:attribute name="class">panelbutton_default</xsl:attribute>
                     <xsl:attribute name="href">#</xsl:attribute>
                    <xsl:attribute name="style">margin-left: 3em;</xsl:attribute>
                    <xsl:attribute name="value">Add</xsl:attribute>
                    <xsl:attribute name="onClick">javascript:addschemakey('<xsl:value-of select="@esb_med_pos"/>');return false;
                    </xsl:attribute>
                </xsl:element>
                <label style='margin-left: 20px;'>Add new Schema key</label>
                <br/>
                <br/>
                <table border="0" style="margin-left: 3em; width: 150px;">
                    <tbody>
                        <tr>
                            <td>
                                <xsl:text>Source</xsl:text>
                            </td>
                            <td>
                                <input type="text" class="esb-edit" size="40"
                                       id="validate_source">
                                    <xsl:attribute name="value">
                                        <xsl:value-of select="@source"/>
                                    </xsl:attribute>
                                </input>
                            </td>
                            <td>
                                <a title="Namespace Editor" class="ns_editor">
                                    <xsl:attribute name="onclick">
                                        javascript:showNameSpaceEditor('<xsl:value-of select="@esb_med_pos"/>','','');
                                    </xsl:attribute>
                                    <xsl:text>&#160;&#160;&#160;&#160;</xsl:text>
                                </a>
                            </td>
                        </tr>
                    </tbody>
                </table>
                <xsl:if test="syn:feature">
                    <p>Xerces-J validator <a href="http://xerces.apache.org/xerces2-j/features.html"
                           target="_blank">features</a> (optional)
                    </p>
                    <br/>
                    <div id="common_table_content">
                        <table border="0" cellpadding="0" cellspacing="0" width="600">
                            <tr>
                                <th>Feature Name</th>
                                <th>Feature Value</th>
                                <th></th>
                            </tr>
                            <xsl:for-each select="syn:feature">
                                <xsl:call-template name="syn-feature-edit"/>
                            </xsl:for-each>
                        </table>
                    </div>
                </xsl:if>
                <input type="button" href="#"
                       onclick="javascript: addmediatorfeature(); return false;"
                       style="margin-left: 3em; width:100px;" value="Add Feature"
                       class="panelbutton_default"/>
                <br/>
                <br/>
                <xsl:for-each select="syn:on-fail">
                    <br/><label style="font-size:13px;">On fail of validation</label><br/>
                    <xsl:if test="count(child::*)=0">
                        <xsl:call-template name="add_mediator_control">
                            <xsl:with-param name="head_pos"
                                        select="concat(concat(@esb_med_pos, '.'), count(child::*))"/>
                        </xsl:call-template>
                    </xsl:if>
                    <div class='indent'><xsl:apply-templates/></div>
                </xsl:for-each>
            </div>
        </div>
    </xsl:template>
     <xsl:template name="schema_keys_template">
        <xsl:param name="value"/>
        <xsl:param name="limit"/>
         <xsl:if test="$value != 0">
             <xsl:call-template name="schema_key_template">
                 <xsl:with-param name="index" select="($limit +1) - $value"/>
             </xsl:call-template>
             <xsl:call-template name="schema_keys_template">
                 <xsl:with-param name="value" select="$value - 1"/>
                 <xsl:with-param name="limit" select="$limit"/>
             </xsl:call-template>
         </xsl:if>
    </xsl:template>
    <xsl:template name="schema_key_template" xmlns:syn="http://ws.apache.org/ns/synapse">
        <xsl:param name="index"/>
        <tr>
            <xsl:if test="//syn:schema[$index]/@key">
                <td>
                    <xsl:text>Key</xsl:text>
                </td>
                <td>
                    <input type="text" size="40" name="schema_key"
                           class="esb-edit_disabled">
                        <xsl:attribute name="id"><xsl:value-of select="concat('schema_key',$index)"/></xsl:attribute>
                        <xsl:attribute name="value">
                            <xsl:value-of select="//syn:schema[$index]/@key"/>
                        </xsl:attribute>
                         <xsl:attribute name="DISABLED"/>
                    </input>
                </td>
                <td>
                    <xsl:element name="img">
                        <xsl:attribute name="style">margin-left: 3em;</xsl:attribute>
                        <xsl:attribute name="src">
                            extensions/core/images/registry_picker.gif</xsl:attribute>
                        <xsl:attribute name="title">Registry
                            Browser</xsl:attribute>
                        <xsl:attribute name="onClick">
                            javascript:schemakeyfromregistry('<xsl:value-of select="$index"/>','<xsl:value-of select="@esb_med_pos"/>');return false;
                        </xsl:attribute>
                    </xsl:element>
                </td>
            </xsl:if>
        </tr>
    </xsl:template>
</xsl:stylesheet>